# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

=pod

=head1 NAME

SendDiff::Cvs::Log - Log/Diff for Cvs

=head1 DESCRIPTION

SendDiff::Cvs::Log is a Perl module which permits to extract the
log/diff from Cvs.

=cut
package SendDiff::Cvs::Log;
@ISA = ('Exporter');
@EXPORT = qw( &createCVSDiffPage ) ;
@EXPORT_OK = qw();
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
my $VERSION = "0.1" ;

use Carp ;
use File::Spec ;
use SendDiff::Array ;
use SendDiff::Error ;
use SendDiff::Factory ;
use SendDiff::Cvs::Run ;

=pod

=back

=head1 CONSTRUCTOR DESCRIPTION

This section contains only the constructor in Factory.pm itself.

=over

=item * new(FACTORY,USERS,LOGFILE)

Constructs a CVS logger.
Takes 3 args:

=over

=item FACTORY is a reference to an object of type SendDiff::HTMLFactory which permits to build a HTML page (scalar).

=item USER is a reference to an associative array which contains the couple login-email for each CVS users (hash ref).

=item LOGFILE specified the path to the file inside which the CVS log are stored. If undef, or if the file does not exist, the content on the standard input will be red.

=back

=cut
sub new($$$): method {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self ;
  if (ref($proto)) {
    %{$self} = %{$proto} ;
  }
  else {
    $self = {'files' => undef,       # list of files on which the CVS works
	     'directory' => undef,   # where the files are
	     'command' => undef,     # type of action from the CVS
	                             # (import,diradd,unknow,commit)
	     'statusmessage' => undef, # list of logs given by CVS
	     'factory' => ($_[0] ||  # reference to the email factory
			   confess('you must supply the factory')),
	     'cvsusers' => ($_[1] || # reference to the CVS user data
			    confess('you must supply the cvs user\'s list')),
	     'logfile' => $_[2],     # filename of the CVS log
	    } ;
  }
  bless( $self, $class );
  return $self;
}

=pod

=head1 METHOD DESCRIPTIONS

This section contains only the methods in Log.pm itself.

=over

=item * generatePage(COMMANDLINE)

Main function (entry point) for the generation of the diff page.
Takes 1 arg:

=over

=item COMMANDLINE is the first command line paramter passed by the CVS system to senddiff.pl.

=back

=cut
sub generatePage($) : method {
  my $self = shift ;
  my $cmdline = shift ;

  verbose("Analyze the log") ;

  # Check the command line passed by CVS
  $self->_scanCommandLine($cmdline) ;

  # Exit silently on 'directory add'.
  exit 0 if ( $self->getCommandType() eq 'diradd' ) ;

  # Exit with error msg on unknow cvs line argument
  if ( $self->getCommandType() eq "unknow" ) {
    safeQuit("Invalid CVS command line arguments '".$cmdline."'");
  }

  # Read the status message from the CVS system
  $self->_readStatusMessage() ;

  if ( $self->getCommandType() eq 'import' ) {
    # Generates a valid list of imported files
    $self->_extractImportedFiles() ;
  }

  # get the name of the commit author and the last revision number
  my ($authorname,$lastrevision) = $self->_extractDataFromRLog() ;

  if ( $self->getCommandType() eq 'import' ) {
    # Initialize the last revision number for imported files
    $self->_setRevisionForImportedFiles($lastrevision) ;
  }

  my @thelogmessage = $self->_extractLogMessage() ;

  verbose("Generate the diff page") ;

  $self->{'factory'}->head($self->{'directory'}) ;

  $self->{'factory'}->description($self->{'directory'},
				  $authorname,
				  $self->{'cvsusers'}{"$authorname"},
				  \@thelogmessage,
				  ($self->getCommandType() eq 'commit'),
				  $self->{'files'}) ;

  if ($self->getCommandType() eq "commit") {

    foreach my $filename (keys %{$self->{'files'}} ) {
      $self->{'factory'}->startfile("$filename") ;

      if ( (!$self->{'files'}{"$filename"}{'old'}) ||
	   (!$self->{'files'}{"$filename"}{'new'}) ) {
	#
	# file add/remove
	#
	# html generation

	# === (1) ====== The header table =============================================
	$self->{'factory'}->filedescription("$filename",
					    $self->{'directory'},
					    $self->{'files'}{"$filename"}{'old'},
					    $self->{'files'}{"$filename"}{'new'},
					    undef) ;
	# === (1) ====== End of the header table ======================================

      }
      else {
	#
	# file update
	#
	$self->_extractDataFromRDiff("$filename") ;

	# html generation

	# === (1) ====== The header table =============================================
	$self->{'factory'}->filedescription("$filename",
					    $self->{'directory'},
					    $self->{'files'}{"$filename"}{'old'},
					    $self->{'files'}{"$filename"}{'old-date'},
					    $self->{'files'}{"$filename"}{'new'},
					    $self->{'files'}{"$filename"}{'new-date'}) ;

	# === (1) ====== End of the header table ======================================

	# === (2) ====== The diff region ==============================================
	$self->_generateFileDiffs("$filename") ;
	# === (2) ====== End of the diff region =======================================

      }

      $self->{'factory'}->stopfile("$filename") ;
    }

  }

  # add the copyright at the end of the html page
  $self->{'factory'}->bottom() ;
}

=pod

=item * _generateFileDiffs(FILE)

Generates the diff section that concern the specified file. The diff must
be extracted from the specified CVS diff's line.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar)

=back

=cut
sub _generateFileDiffs($) : method {
  my $self = shift ;
  my $filename = shift ;
  foreach my $diff ( @{$self->{'files'}{"$filename"}{'diffs'}} ) {
    if ( isarray($diff) ) {
      my $head = shift @{$diff} ;

      $self->{'factory'}->startfilediff("$filename","$head") ;

      foreach my $diffline (@{$diff}) {

	if ( $diffline =~ /^=(.*)$/ ) {
	  $self->{'factory'}->diff_diff("$filename","$1") ;
	}
	elsif ( $diffline =~ /^\+(.*)$/ ) {
	  $self->{'factory'}->diff_add("$filename","$1") ;
	}
	elsif ( $diffline =~ /^\-(.*)$/ ) {
	  $self->{'factory'}->diff_del("$filename","$1") ;
	}

      }
    }

    $self->{'factory'}->stopfilediff($filename) ;
  }
}

=pod

=item * _extractLogMessage()

Extracts the log message from the status stream.

=cut
sub _extractLogMessage() : method {
  my $self = shift ;
  my @status = @{$self->{'statusmessage'}} ;

  while ( (@status) && ( $status[0] !~ /^Log Message:/) ) {
    shift @status ;
  }

  if ( @status ) {
    $status[0] =~ s/^Log Message:// ;
    if ( $self->getCommandType() eq 'import' ) {
      my @s = () ;
      while ( (@status) && ( $status[0] !~ /^Release Tags:/) ) {
	push @s, shift @status ;
      }
      push @s, shift @status if (@status);
      @status = @s ;
    }
  }
  return @status ;
}

=pod

=item * _extractDataFromRDiff(FILE)

Replies some additional data about the specified file. The data
are extracted from a CVS rdiff call.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar)

=back

=cut
sub _extractDataFromRDiff($) : method {
  my $self = shift ;
  my $filename = shift ;
  my $lfilename = File::Spec->catfile($self->{'directory'},"$filename") ;

  my @rdiff = cvsrun_rdiff("$lfilename",
			   $self->{'files'}{"$filename"}{'old'},
			   $self->{'files'}{"$filename"}{'new'}) ;

  safeQuit( "invalid diff result from cvs: empty result" )
    unless (@rdiff) ;

  my $line = $rdiff[0] ;
  safeQuit( "invalid diff format from cvs: no index header" )
    unless (($line)&&( $line =~ /^Index:\s*\Q$lfilename\E/ )) ;

  my $indiff = undef ;
  my @currentdiff = () ;

  for(my $i=0; $i<@rdiff; $i++) {
    $line = $rdiff[$i] ;
    if ($line) {
      if (!@currentdiff) {
	# treat the header
	if ( $line =~ /^\Q---\E\s+\Q$lfilename\E(?::[^\s]+)?\s+(.*)\s*$/ ) {
	  $self->{'files'}{"$filename"}{'old-date'} = "$1" ;
	  $self->{'files'}{"$filename"}{'old-date'} =~ s/\s+/ /g ;
	  chomp($self->{'files'}{"$filename"}{'old-date'}) ;
	}
	elsif ( $line =~ /^\Q+++\E\s+\Q$lfilename\E(?::[^\s]+)?\s+(.*)\s*$/ ) {
	  $self->{'files'}{"$filename"}{'new-date'} = "$1" ;
	  $self->{'files'}{"$filename"}{'new-date'} =~ s/\s+/ /g ;
	  chomp($self->{'files'}{"$filename"}{'new-date'}) ;
	}
	elsif ( $line =~ /^@@\s*(.+?)\s*@@\s*$/ ) {
	  @currentdiff = ( "$1" ) ;
	}
      }
      else {
	# treat the diff description
	if ( $line =~ /^([ +-])(.*)$/ ) {
	  my $s = ("$1" eq ' ') ? '=' : "$1" ;
	  push @currentdiff, "$s$2" ;
	}
	else {
	  # line from another diff description
	  if ( ! $self->{'files'}{"$filename"}{'diffs'} ) {
	    $self->{'files'}{"$filename"}{'diffs'} = [] ;
	  }
	  push @{$self->{'files'}{"$filename"}{'diffs'}}, \@currentdiff ;
	  @currentdiff = () ;
	  $i -- ;
	}
      }
    }
  }

  if (@currentdiff) {
    if ( ! $self->{'files'}{"$filename"}{'diffs'} ) {
      $self->{'files'}{"$filename"}{'diffs'} = [] ;
    }
    push @{$self->{'files'}{"$filename"}{'diffs'}}, \@currentdiff ;
  }

  $self->{'rdiff'} = \@rdiff ;
}

=pod

=item * _extractDataFromRLog()

Replies the author and the last revision number.

=cut
sub _extractDataFromRLog() : method {
  my $self = shift ;
  my ($author,$revision) = ('','') ;
  # Get informations about the first file
  if ( %{$self->{'files'}} ) {
    my @filenames = keys %{$self->{'files'}} ;
    my $filename = $filenames[0] ;
    my $newversion = $self->{'files'}{"$filename"}{'new'} || '' ;

    # get the author of this commit or this export
    my @lines ;
    if ( $filename eq '-' ) {
      @lines = cvsrun_rlog_dir($self->{'directory'}) ;
    }
    else {
      @lines = cvsrun_rlog_file(File::Spec->catfile($self->{'directory'},"$filename"),"$newversion") ;
    }

    $self->{'rlog'} = \@lines ;

    foreach my $line ( @lines ) {
      if ( $line =~ /author:\s*([^;]+);/ ) {
	$author = "$1" ;
      }
      if ( $line =~ /head:\s*([^\s]+)\s*/ ) {
	$revision = "$1" ;
      }
      if (($author)&&($revision)) {
	return ($author,$revision) ;
      }
    }
  }
  return ($author,$revision) ;
}

=pod

=item * _setRevisionForImportedFiles(REVISION)

Extracts the filenames of the imported files.
Takes 1 arg:

=over

=item REVISION is the revision number to attach to imported files

=back

=cut
sub _setRevisionForImportedFiles($) : method {
  my $self = shift ;
  my $revision = shift ;
  foreach my $file (keys %{$self->{'files'}}) {
    $self->{'files'}{"$file"}{'new'} = $revision ;
  }
}


=pod

=item * _extractImportedFiles()

Extracts the filenames of the imported files.

=cut
sub _extractImportedFiles() : method {
  my $self = shift ;
  my @status = @{$self->{'statusmessage'}} ;

  while ( (@status) && ( $status[0] !~ /^Release Tags:/) ) {
    shift @status ;
  }
  if ( @status ) {
    shift @status ;
    my $dir = $self->{'directory'} ;
    while (@status) {
      my $line = shift @status ;
      if ( $line =~ /^N\s+\Q$dir\E[\/\\](.+)\s*$/ ) {
	$self->{'files'}{"$1"}{'old'} = undef ;
	$self->{'files'}{"$1"}{'new'} = undef ;
      }
    }
  }
}

=pod

=item * _scanCommandLine(TEXT)

Split the cvs argument into the directory
where the commit has been done, and the list
of modified files.
Check the kind of the command passed to cvs :
commit, import, or directory add.
Takes 1 arg:

=over

=item TEXT is the first command line paramter passed by the CVS system to senddiff.pl.

=back

=cut
sub _scanCommandLine($) : method {
  my $self = shift ;
  my @files = split /[\s,]+/, ($_[0]||'') ;
  my $directory = shift @files ;
  my $command ;
  $self->{'files'} = {} ;

  if ( (@files == 3) && ($files[0] eq "-") ) {
    if ( ($files[1] eq "Imported") && ($files[2] eq "sources") ) {
      $command = "import" ;
    }
    elsif ( ($files[1] eq "New") && ($files[2] eq "directory") ) {
      $command = "diradd" ;
    }
    else {
      $command = "unknow" ;
    }
  }
  else {
    $command = "commit" ;
    for(my $i=0; $i<@files; $i=$i+3) {
      my $filename = $files[$i];
      my $oldversion = $files[$i+1];
      my $newversion = $files[$i+2];
      if ((!$oldversion)||(uc($oldversion) eq "NONE")) {
	$self->{'files'}{"$filename"}{'old'} = undef ;
      }
      else {
	$self->{'files'}{"$filename"}{'old'} = "$oldversion" ;
      }
      if ((!$newversion)||(uc($newversion) eq "NONE")) {
	$self->{'files'}{"$filename"}{'new'} = undef ;
      }
      else {
	$self->{'files'}{"$filename"}{'new'} = "$newversion" ;
      }
    }
  }
  $self->{'directory'} = $directory ;
  $self->{'command'} = $command ;
}

=pod

=item * getCommandType()

Replies the type of action sended by CVS.

=cut
sub getCommandType() : method {
  my $self = shift ;
  return $self->{'command'} ;
}

=pod

=item * _readStatusMessage()

Reads the status from the CVS system.

=cut
sub _readStatusMessage() : method {
  my $self = shift ;
  # Save in a table the lines of the cvs log description
  my @cvslogmsg ;
  if (($self->{'logfile'})&&
      (-f $self->{'logfile'})&&
      (-r $self->{'logfile'})) {
    local *FID ;
    open(*FID, "< ".$self->{'logfile'})
      or safeQuit($self->{'logfile'}.": $!") ;
    @cvslogmsg = <FID> ;
    close(*FID) ;
  }
  else {
    @cvslogmsg = <STDIN> ;
  }
  # remove the 2 two first line from the cvs log message
  shift @cvslogmsg ;
  shift @cvslogmsg ;
  shift @cvslogmsg ;
  # save the log
  $self->{'statusmessage'} = \@cvslogmsg ;
}


1;
__END__

=back
