# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

=pod

=head1 NAME

SendDiff::Cvs::Run - Running Cvs

=head1 DESCRIPTION

SendDiff::Cvs::RUN is a Perl module which permits to run
the CVs program.

=cut
package SendDiff::Cvs::Run;
@ISA = ('Exporter');
@EXPORT = qw( &getCVSBinary &setCVSBinary
	      &cvsrun_rlog_dir &cvsrun_rlog_file
	      &cvsrun_rdiff ) ;
@EXPORT_OK = qw();
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
my $VERSION = "0.1" ;

use File::Spec ;
use SendDiff::Error ;
use SendDiff::Array ;
use SendDiff::Cvs ;

# --- some installation dependant variables ---
my %CVS_CMD = ( 'cvs'      => '/usr/bin/cvs',           # the CVS binary
		'options'  => ['-q','-n'],              # the CVS general options
		'root'     => ['-d','$CVSROOT'],        # the CVSROOT directory parameter
		                                        # $CVSROOT will be replaced by the
		                                        # right CVSROOT directory
		'rlog'     => ['-f','rlog','-N'],       # CVS parameters to obtain a log
		'rdiff'    => ['rdiff','-l','-u'],      # CVS parameters to obtain a diff
	      ) ;
# ---------------------------------------------

=pod

=head1 FUNCTION DESCRIPTIONS

This section contains only the functions in Run.pm itself.

=over

=item * getCVSBinary()

Replies the path to the CVS binary file.

=cut
sub getCVSBinary() {
  return $CVS_CMD{'cvs'} ;
}

=pod

=item * setCVSBinary(PATH)

Sets the path to the CVS binary.
Takes 1 arg:

=over

=item PATH is the path to the CVS ninary (scalar).

=back

=cut
sub setCVSBinary($) {
  if ( $_[0] ) {
    safeQuit("the executable file $_[0] was not found")
      unless ( -f "$_[0]" ) ;
    safeQuit("the executable file $_[0] has not the running right")
      unless ( -x "$_[0]" ) ;

    $CVS_CMD{'cvs'} = "$_[0]" ;

    my $cvsroot = getLocalCVSROOT() ;
    foreach my $key (keys %CVS_CMD) {
      if ( isarray($CVS_CMD{"$key"}) ) {
	for(my $i=0; $i<@{$CVS_CMD{"$key"}}; $i++) {
	  $CVS_CMD{"$key"}[$i] =~ s/\$CVSROOT/$cvsroot/g ;
	}
      }
      else {
	$CVS_CMD{"$key"} =~ s/\$CVSROOT/$cvsroot/g ;
      }
    }
  }
}

=pod

=item * cvsrun_rlog_dir(PATH)

Runs the rlog CVS command on a directory and replies the log's lines.
Takes 1 arg:

=over

=item PATH is the path of the directory (scalar).

=back

=cut
sub cvsrun_rlog_dir($) {
  my @cvscmd = mergearray( $CVS_CMD{'cvs'},
			   $CVS_CMD{'options'},
			   $CVS_CMD{'root'},
			   $CVS_CMD{'rlog'} ) ;
  push @cvscmd, File::Spec->catfile("$_[0]","") ;
  local *LOG ;
  open(*LOG, join(' ',@cvscmd)." |")
    or safeQuit("unable to run the CVS rlog command: $!") ;
  my @lines = <LOG> ;
  close LOG ;
  return @lines ;
}

=pod

=item * cvsrun_rlog_file(PATH,[REVISION])

Runs the rlog CVS command on a directory and replies the log's lines.
Takes 2 args:

=over

=item PATH is the path of the file (scalar).

=item REVISION is the desired revision number.

=back

=cut
sub cvsrun_rlog_file($;$) {
  my @cvscmd = mergearray( $CVS_CMD{'cvs'},
			   $CVS_CMD{'options'},
			   $CVS_CMD{'root'},
			   $CVS_CMD{'rlog'} ) ;
  push @cvscmd, "-r$_[1]" if ( $_[1] ) ;
  push @cvscmd, "$_[0]" ;
  local *LOG ;
  open(*LOG, join(' ',@cvscmd)." |")
    or safeQuit("unable to run the CVS rlog command: $!") ;
  my @lines = <LOG> ;
  close LOG ;
  return @lines ;
}

=pod

=item * cvsrun_rdiff(FILE,OLDVERSION,NEWVERSION)

Runs the rdiff CVS command on a directory and replies the log's lines.
Takes 3 args:

=over

=item FILE is the path of the file on which the diff must be done (scalar).

=item OLDVERSION is the old version reference (scalar).

=item NEWVERSION is the new version reference (scalar).

=back

=cut
sub cvsrun_rdiff($$$) {
  my $filename = shift ;
  my $oldversion = shift ;
  my $newversion = shift ;
  my @cvscmd = mergearray( $CVS_CMD{'cvs'},
			   $CVS_CMD{'options'},
			   $CVS_CMD{'root'},
			   $CVS_CMD{'rdiff'},
			   "-r$oldversion",
			   "-r$newversion",
			   "$filename" ) ;
  local *DIFF ;
  open(*DIFF, join(' ',@cvscmd)." |")
    or safeQuit("unable to run the CVS rdiff command: $!") ;
  my @lines = <DIFF> ;
  close DIFF ;
  return @lines ;
}

1;
__END__

=back
