# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

=pod

=head1 NAME

SendDiff::Email - SendDiff's Mailer

=head1 DESCRIPTION

SendDiff::Email is a Perl module which implementes a mailer
based on SMTP, sendmail or Mail::Mailer depending on the
initialization.

=cut
package SendDiff::Email;
@ISA = ('Exporter');
@EXPORT = qw( &mailer &initMailer ) ;
@EXPORT_OK = qw();
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
my $VERSION = "0.1" ;

use File::Path ;
use File::Temp qw/ tempfile /;
use SendDiff::Error ;

# ------------------- global variables --------
my $CURRENT_MAILER = undef ;

=pod

=head1 FUNCTION DESCRIPTIONS

This section contains only the functions in Email.pm itself.

=over

=item * mailer()

Replies the current mailer

=cut
sub mailer() {
  return $CURRENT_MAILER ;
}

=pod

=item * initMailer(SENDER,METHOD,[PARAM])

Initializes the current mailer.
Takes 3 args:

=over

=item SENDER is the email of the sender (scalar).

=item METHOD is the method identifier (scalar). It must be one of I<smtp>, I<sendmail> and I<mail>.

=item PARAM is the parameter (scalar) which corresponds to the desired method: the SMTP server's address, the sendmail binary path.

=cut
sub initMailer($$;$) {
  my $email = shift ;
  $CURRENT_MAILER = SendDiff::Email->new($email) ;
  if (($_[0])&&($_[0] eq 'spool')) {
    $CURRENT_MAILER->initAsyncMail($_[1]) ;
  }
  elsif (($_[0])&&($_[0] eq 'smtp')) {
    $CURRENT_MAILER->initSMTPMail($_[1]) ;
  }
  elsif (($_[0])&&($_[0] eq 'sendmail')) {
    $CURRENT_MAILER->initSendmailMail($_[1]) ;
  }
  else {
    $CURRENT_MAILER->initMailerMail() ;
  }
  return $CURRENT_MAILER ;
}

=pod

=back

=head1 CONSTRUCTOR DESCRIPTION

This section contains only the constructor in Email.pm itself.

=over

=item * new(SENDER)

Constructs a mailer without any sdending method set.
Takes 1 arg:

=over

=item SENDER is the email of the sender (scalar).

=back

=cut
sub new($): method {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self ;
  if (ref($proto)) {
    %{$self} = %{$proto} ;
  }
  else {
    $self = { 'sendmail' => { 'cmd' => "/usr/sbin/sendmail",
			      'options' => "-i",
			    },
	      'smtp' => '127.0.0.1',
	      'method' => '',
	      'recipients' => [],
	      'sender' => $_[0] || 'senddiff@localhost',
	      'spool' => '/var/spool/senddiff',
	    } ;
  }
  bless( $self, $class );
  return $self;
}

=pod

=back

=head1 METHOD DESCRIPTIONS

This section contains only the methods in Email.pm itself.

=over

=item * initASynMail(DIRECTORY)

Initializes the mail service with an asynchronous feature.
Takes 1 arg:

=over

=item DIRECTORY is the path to the spool (scalar).

=back

=cut
sub initAsyncMail($) : method {
  my $self = shift ;
  my $spool = shift ;

  if ( ! $spool ) {
    safeQuit("you must specify the spool directory") ;
  }
  $self->{'spool'} = $spool ;
  $self->{'method'} = 'spool' ;
}

=pod

=item * initSMTPMail(ADDRESS)

Initializes the mail service with the SMTP feature.
Takes 1 arg:

=over

=item ADDRESS is the address of the SMTP server (scalar).

=back

=cut
sub initSMTPMail($) : method {
  my $self = shift ;
  my $smtp = shift ;

  chomp($smtp);
  if ( ! $smtp ) {
    safeQuit("you must specify the SMTP server") ;
  }
  eval( 'require Net::SMTP ;' ) ;
  if ( $@ ) {
    safeQuit("unable to find the Perl package Net::SMTP needed to contact the SMTP server");
  }
  $self->{'smtp'} = $smtp ;
  $self->{'method'} = 'smtp' ;
}

=pod

=item * initSendmailMail([FILE])

Initializes the mail service with the Sendmail feature.
Takes 1 arg:

=over

=item FILE is the path to the sendmail binary (scalar).

=back

=cut
sub initSendmailMail(;$) : method {
  my $self = shift ;
  my $sendmail = shift ;

  if (defined($sendmail)) {
    chomp($sendmail) ;
    if ( $sendmail ) {
      $self->{'sendmail'}{'cmd'} = $sendmail ;
    }
  }
  if ( ! -x $self->{'sendmail'}{'cmd'} ) {
    safeQuit("unable to find the sendmail binary: ".$self->{'sendmail'}{'cmd'}) ;
  }
  $self->{'method'} = 'sendmail' ;
}

=pod

=item * initMailerMail()

Initializes the mail service with the default Mailer feature.

=cut
sub initMailerMail() : method {
  my $self = shift ;
  eval( 'require Mail::Mailer ;');
  if ( $@ ) {
    safeQuit("unable to find the Perl package Mail::Mailer needed to use the default email deliver");
  }
  $self->{'method'} = 'mailer' ;
}

=pod

=item * setRecipients([ADDRESS, [ADDRESS...]])

Initializes the recipients addresses.
Takes n args:

=over

=item ADDRESS is the email address of the recipient (scalar).

=back

=cut
sub setRecipients(@) : method {
  my $self = shift ;
  $self->{'recipients'} = [] ;
  foreach my $recipient (@_) {
    if (($recipient)&&($recipient =~ /^[0-9a-zA-Z._-]+@[0-9a-zA-Z._-]+$/)) {
      push @{$self->{'recipients'}}, "$recipient" ;
    }
  }
}

=pod

=item * hasRecipients()

Replies if this mailer has recipient's addresses.

=cut
sub hasRecipients() : method {
  my $self = shift ;
  return ((exists($self->{'recipients'}))&&
	  (defined($self->{'recipients'}))&&
	  (@{$self->{'recipients'}})) ;
}

=pod

=item * sendemail(SUBJECT,TEXT)

Send the specified email.
Takes 2 args:

=over

=item SUBJECT is the subject of the email (scalar).

=item TEXT is the body content of the email (scalar).

=back

=cut
sub sendemail($$) {
  my $self = shift ;

  if ( $self->hasRecipients() ) {

    if ( $self->{'method'} eq 'spool' ) {
      verbose("Differ the notification to ".join(',',@{$self->{'recipients'}})) ;

      $self->_send_spool(@_,@{$self->{'recipients'}}) ;
    }
    else {
      verbose("Send notification to ".join(',',@{$self->{'recipients'}})) ;

      if ( $self->{'method'} eq 'smtp' ) {
	$self->_send_smtp(@_,@{$self->{'recipients'}}) ;
      }
      elsif ( $self->{'method'} eq 'sendmail' ) {
	$self->_send_sendmail(@_,@{$self->{'recipients'}}) ;
      }
      else {
	$self->_send_mailer(@_,@{$self->{'recipients'}}) ;
      }
    }
  }

}

=pod

=item * _send_spool(SUBJECT,TEXT,[EMAIL...])

Send asynchronously the specified email.
Takes n args:

=over

=item SUBJECT is the subject of the email (scalar).

=item TEXT is the body content of the email (scalar).

=item EMAIL is the address of a recipient (scalar).

=back

=cut
sub _send_spool($$@) {
  my $self = shift ;
  my $subject = shift ;
  my $body = shift ;

  if (!(-d "$self->{'spool'}")) {
    eval { mkpath("$self->{'spool'}"); } ;
    if ( $@ ) {
      safeQuit("$self->{'spool'}: $@") ;
    }
  }
  if (!(-w "$self->{'spool'}")) {
    safeQuit("$self->{'spool'}: $!") ;
  }
  else {

    my ($handle,$filename) = tempfile( "notifyXXXXXX", DIR=>$self->{'spool'} );

    if (!$handle) {
      safeQuit("$self->{'spool'}: unable to create a notification file") ;
    }

    $subject =~ s/[\r\n]+/ /g ;

    print $handle "From: ".$self->{'sender'}."\n" ;
    print $handle "To: ".join(',',@_)."\n" ;
    print $handle "Subject: $subject\n" ;
    print $handle "$body" ;
    close $handle ;

  }
}

=pod

=item * _send_smtp(SUBJECT,TEXT,[EMAIL...])

Send the specified email.
Takes n args:

=over

=item SUBJECT is the subject of the email (scalar).

=item TEXT is the body content of the email (scalar).

=item EMAIL is the address of a recipient (scalar).

=back

=cut
sub _send_smtp($$@) {
  my $self = shift ;
  my $subject = shift ;
  my $body = shift ;

  my $smtp = Net::SMTP->new($self->{'smtp'}) ;

  $smtp->mail($self->{'sender'}) ;
  $smtp->to(@_) ;

  $subject =~ s/[\r\n]+/ /g ;

  $smtp->data();
  $smtp->datasend("From: ".$self->{'sender'}."\n") ;
  $smtp->datasend("To: ".join(',',@_)."\n") ;
  $smtp->datasend("Subject: $subject\n") ;
  $smtp->datasend("Content-Type: text/html; charset=iso-8859-1\n") ;
  $smtp->datasend("Content-Transfer-Encoding: 8bit\n") ;
  $smtp->datasend("\n");
  $smtp->datasend($body);
  $smtp->dataend();

  $smtp->quit;
}

=pod

=item * _send_sendmail(SUBJECT,TEXT,[EMAIL...])

Send the specified email.
Takes n args:

=over

=item SUBJECT is the subject of the email (scalar).

=item TEXT is the body content of the email (scalar).

=item EMAIL is the address of a recipient (scalar).

=back

=cut
sub _send_sendmail($$@) {
  my $self = shift ;
  my $subject = shift ;
  my $body = shift ;

  $subject =~ s/[\r\n]+/ /g ;

  # send the mail with my own configuration
  local *MAIL ;
  unless ( open(*MAIL, join( ' ',
			     "|",
			     $self->{'sendmail'}{'cmd'},
			     $self->{'sendmail'}{'options'},
			     @_ ) ) ) {
    safeQuit("Cannot send a mail with sendmail: $!\n") ;
  }
  print MAIL join("\n",
		  "From: ".$self->{'sender'},
		  "To: ".join(',',@_),
		  "Subject: $subject",
		  "Content-Type: text/html; charset=iso-8859-1",
		  "Content-Transfer-Encoding: 8bit",
		  "",
		  $body ) ;
  close(*MAIL) ;
}

=pod

=item * _send_mailer(SUBJECT,TEXT,[EMAIL...])

Send the specified email.
Takes n args:

=over

=item SUBJECT is the subject of the email (scalar).

=item TEXT is the body content of the email (scalar).

=item EMAIL is the address of a recipient (scalar).

=back

=cut
sub _send_mailer($$@) {
  my $self = shift ;
  my $subject = shift ;
  my $body = shift ;

  $subject =~ s/[\r\n]+/ /g ;

  my $mailer = Mail::Mailer->new();

  if ( ! $mailer->open( { 'From' => $self->{'sender'},
			  'To' => \@_,
			  'Subject' => "$subject",
			  'Content-Type' => "text/html; charset=iso-8859-1\n",
			  'Content-Transfer-Encoding' => "8bit",
			}
		      ) ) {
    safeQuit("can't open the built-in mailer: $!\n");
  }
  else {
    print $mailer "$body" ;
    $mailer->close();
  }
}

1;
__END__

=back
