# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

=pod

=head1 NAME

SendDiff::Factory - Factory for a email notification page.

=head1 DESCRIPTION

SendDiff::Factory is a Perl module which permits to create an
page to send by email.

=cut
package SendDiff::Factory;
@ISA = ('Exporter');
@EXPORT = qw() ;
@EXPORT_OK = qw();
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
my $VERSION = "0.1" ;

use Carp ;
use SendDiff::Array ;

=pod

=back

=head1 CONSTRUCTOR DESCRIPTION

This section contains only the constructor in Factory.pm itself.

=over

=item * new()

Constructs a factory.

=cut
sub new(): method {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self ;
  if (ref($proto)) {
    %{$self} = %{$proto} ;
  }
  else {
    $self = { 'content' => '',
	      'subject' => '' } ;
  }
  bless( $self, $class );
  return $self;
}

=pod

=back

=head1 METHOD DESCRIPTIONS

This section contains only the methods in Factory.pm itself.

=over

=item * addtocontent(TEXT, [TEXT...])

Add the specified text to the content of the generated page.

=cut
sub addtocontent($) : method {
  my $self = shift ;
  foreach my $e (@_) {
    $self->{'content'} .= "$e" if ( $e ) ;
  }
  return 1 ;
}

=pod

=item * getcontent()

Replies the content generated by this factory.

=cut
sub getcontent() : method {
  my $self = shift ;
  return scalar($self->{'content'}) ;
}

=pod

=item * getsubject()

Replies a string which briefly describe the content.

=cut
sub getsubject() : method {
  my $self = shift ;
  return scalar($self->{'subject'}) ;
}

=pod

=item * head(FILE)

Generates the beginning of the email.
Takes 1 arg:

=over

=item FILE is the directory where the action was done (scalar)

=back

=cut
sub head($) : method {
  my $self = shift ;
  my $directory = shift ;
  $self->{'subject'} = "cvs changes in $directory" ;
}

=pod

=item * bottom()

Generates the end of the email.

=cut
sub bottom() : method {
}

=pod

=item * description(FILE,AUTHOR,EMAIL,LOG,COMMITFLAG,FILES)

Fill the page with the description of the change.
Takes 6 args:

=over

=item FILE is the directory where the action was done (scalar)

=item AUTHOR is the name of the author (scalar)

=item EMAIL is the email of author (scalar)

=item LOG is the log message given by the author (scalar).

=item COMMITFLAG indicates if the files was commit or not.

=item FILES is the associative array of the files to display

=cut
sub description($$$\@$\%) : method {
  my $self = shift ;
  my $directory = shift ;
  my $authorname = shift ;
  my $email = shift ;
  my $log = shift ;
  my $commitflag = shift ;
  $self->listoffiles($directory,$commitflag,shift) ;
  $self->logmessage($authorname,$email,@{$log}) ;
}

=pod

=item * listoffiles(FILE,COMMITFLAG,FILES)

Fills the mail body with the list of files.
If a filename starts with a tabulation, it is
a filename, otherwhise it is a comment.
Takes n args:

=over

=item FILE is the directory where the action was done (scalar)

=item COMMITFLAG indicates if the files was commit or not.

=item FILES is the associative array of the files to display

=cut
sub listoffiles($$\%) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * logmessage(AUTHOR,EMAIL,LOG,[LOG...])

Fills the mail body with the log message.
Takes n args:

=over

=item AUTHOR is the name of the author (scalar)

=item EMAIL is the email of author (scalar)

=item LOG is one line of the log message (scalar).

=cut
sub logmessage($$@) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * startfile(FILE)

Start the diff section for the specified file.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar).

=cut
sub startfile($) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * stopfile(FILE)

Stop the diff section for the specified file.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar).

=cut
sub stopfile($) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * filedescription(FILE,DIRECTORY,OLDVERSION,OLDDATE,NEWVERSION,NEWDATE)

Fills the mail body with the description of the specified file.
Takes 6 args:

=over

=item FILE is the name of the file (scalar).

=item DIRECTORY is the directory where the file was (scalar).

=item OLDVERSION is the previous version of the file in CVS (scalar or undef).

=item OLDDATE is the date of the previous version (scalar or undef).

=item NEWVERSION is the current version of the file in CVS (scalar or undef).

=item NEWDATE is the date of the current version (scalar or undef).

=cut
sub filedescription($$$$$$) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * startfilediff(FILE,HEAD)

Start the diff for the specified file.
Takes n args:

=over

=item FILE is the name of the file (scalar).

=item HEAD is the text which describe this diff (scalar).

=cut
sub startfilediff($$) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * stopfilediff(FILE)

Stop the diff for the specified file.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar).

=cut
sub stopfilediff($) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * diff_diff(FILE,TEXT)

Add a diff message which describes a change for the specified file.
Takes 2 args:

=over

=item FILE is the name of the file (scalar).

=item TEXT is the diff text to add (scalar).

=cut
sub diff_diff($$) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * diff_del(FILE,TEXT)

Add a diff message which describes a deletion for the specified file.
Takes 2 args:

=over

=item FILE is the name of the file (scalar).

=item TEXT is the diff text to add (scalar).

=cut
sub diff_del($$) : method {
  confess("you must overwrite this method into a subclass") ;
}

=pod

=item * diff_add(FILE,TEXT)

Add a diff message which describes an addition for the specified file.
Takes 2 args:

=over

=item FILE is the name of the file (scalar).

=item TEXT is the diff text to add (scalar).

=cut
sub diff_add($$) : method {
  confess("you must overwrite this method into a subclass") ;
}

1;
__END__

=back
