# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

=pod

=head1 NAME

SendDiff::Factory::HTML - Factory for an HTML email notification page.

=head1 DESCRIPTION

SendDiff::Factory::HTML is a Perl module which permits to create an
HTML page to send by email.

=cut
package SendDiff::Factory::HTML;
@ISA = ('SendDiff::Factory');
@EXPORT = qw() ;
@EXPORT_OK = qw();
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
my $VERSION = "0.1" ;

use File::Spec;
use SendDiff::Factory ;
use SendDiff::Release ;

# ------------------------------------------------------------------------
#                       Text to HTML translation table

# The characters which are displayed for each HTML entity (except &amp; &gt; &lt; &quot; )
my %HTML_ENTITY_CHARS = (  'Ocirc'        => '',
			   'szlig'        => '',
			   'micro'        => '',
			   'para'         => '',
			   'not'          => '',
			   'sup1'         => '',
			   'oacute'       => '',
			   'Uacute'       => '',
			   'middot'       => '',
			   'ecirc'        => '',
			   'pound'        => '',
			   'scaron'       => '',
			   'ntilde'       => '',
			   'igrave'       => '',
			   'atilde'       => '',
			   'thorn'        => '',
			   'Euml'         => '',
			   'Ntilde'       => '',
			   'Auml'         => '',
			   'plusmn'       => '',
			   'raquo'        => '',
			   'THORN'        => '',
			   'laquo'        => '',
			   'Eacute'       => '',
			   'divide'       => '',
			   'Uuml'         => '',
			   'Aring'        => '',
			   'ugrave'       => '',
			   'Egrave'       => '',
			   'Acirc'        => '',
			   'oslash'       => '',
			   'ETH'          => '',
			   'iacute'       => '',
			   'Ograve'       => '',
			   'Oslash'       => '',
			   'frac34'       => '3/4',
			   'Scaron'       => '',
			   'eth'          => '',
			   'icirc'        => '',
			   'ordm'         => '',
			   'ucirc'        => '',
			   'reg'          => '',
			   'tilde'        => '~',
			   'aacute'       => '',
			   'Agrave'       => '',
			   'Yuml'         => '',
			   'times'        => '',
			   'deg'          => '',
			   'AElig'        => '',
			   'Yacute'       => '',
			   'Otilde'       => '',
			   'circ'         => '^',
			   'sup3'         => '',
			   'oelig'        => '',
			   'frac14'       => '1/4',
			   'Ouml'         => '',
			   'ograve'       => '',
			   'copy'         => '',
			   'shy'          => '',
			   'iuml'         => '',
			   'acirc'        => '',
			   'iexcl'        => '',
			   'Iacute'       => '',
			   'Oacute'       => '',
			   'ccedil'       => '',
			   'frac12'       => '1/2',
			   'Icirc'        => '',
			   'eacute'       => '',
			   'egrave'       => '',
			   'euml'         => '',
			   'Ccedil'       => '',
			   'OElig'        => '',
			   'Atilde'       => '',
			   'ouml'         => '',
			   'cent'         => '',
			   'Aacute'       => '',
			   'sect'         => '',
			   'Ugrave'       => '',
			   'aelig'        => '',
			   'ordf'         => '',
			   'yacute'       => '',
			   'Ecirc'        => '',
			   'auml'         => '',
			   'macr'         => '',
			   'iquest'       => '',
			   'sup2'         => '',
			   'Ucirc'        => '',
			   'aring'        => '',
			   'Igrave'       => '',
			   'yen'          => '',
			   'uuml'         => '',
			   'otilde'       => '',
			   'uacute'       => '',
			   'yuml'         => '',
			   'ocirc'        => '',
			   'Iuml'         => '',
			   'agrave'       => '',
			) ;
# ------------------------------------------------------------------------

=pod

=back

=head1 CONSTRUCTOR DESCRIPTION

This section contains only the constructor in HTML.pm itself.

=over

=item * new()

Constructs an HTML factory.

=cut
sub new(): method {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = $class->SUPER::new( @_ ) ;

  # ------------------------------------------------------------------------
  #                       Default colors in html mode
  $self->{'colors'}{'fileHeadRegion'} = '#ffff66' ;  # color of the backround for the header
                                                     # of the file diff table
  $self->{'colors'}{'diffHeadRegion'} = '#99ff99' ;  # color of the background for the header
                                                     # of the diff region
  $self->{'colors'}{'diffAddColor'}   = '#0000ff' ;  # color of the added lines
  $self->{'colors'}{'diffDelColor'}   = '#ff0000' ;  # color of the removed lines
  $self->{'colors'}{'descriptionBG'}  = '#cccccc' ;  # color for the description background.
  $self->{'colors'}{'authorColor'}    = '#0000ff' ;  # color for the author name
  # ------------------------------------------------------------------------

  bless( $self, $class );
  return $self;
}

=pod

=item * text2html(TEXT, [TEXT...])

Replies the translation into HTML of the specified texts.
Takes n args.

=cut
sub text2html(@) : method {
  my $self = shift ;
  my $result = '' ;
  foreach my $txt (@_) {
    if ( $txt ) {
      chomp($txt) ;

      # the first step is to replace the '&'
      # and then the '<' and '>' character.
      $txt =~ s/&/&amp;/g ;
      $txt =~ s/</&lt;/g ;
      $txt =~ s/>/&gt;/g ;
      $txt =~ s/\"/&quot;/g ;

      # FIXME: we need to replace multiple space only
      $txt =~ s/ /&nbsp;/g  ;

      foreach my $html (keys %HTML_ENTITY_CHARS ) {
	my $text = $HTML_ENTITY_CHARS{"$html"} ;
	$txt =~ s/\Q$text\E/\&$html;/g ;
      }

      chomp($txt) ;
      $result .= $txt ;
    }
  }
  return $result ;
}

=pod

=item * head(FILE)

Generates the beginning of the email.
Takes 1 arg:

=over

=item FILE is the directory where the action was done (scalar)

=back

=cut
sub head($) : method {
  my $self = shift ;
  $self->SUPER::head(@_);
  $self->addtocontent("<html><head><title>cvs changes in $_[0]</title></head><body>\n") ;
}

=pod

=item * bottom()

Generates the end of the email.

=cut
sub bottom() : method {
  my $self = shift ;
  $self->addtocontent("<br><center>\n",
		      "<br><br><hr width='60%' size='1'><br>\n",
		      "This page has been generated by the <i>'senddiff'</i> tools<br>\n",
		      "<small>(version ",
		      getVersionNumber(),
		      "&nbsp;-&nbsp;",
		      getVersionDate(),
		      ").</small><br>\n",
		      "<a href=\"",
		      getMainURL(),
		      "\">",
		      getMainURL(),
		      "</a><br>\n" ) ;
  my $authors = '' ;
  my %authors = getAuthors() ;
  foreach my $email (keys %authors ) {
    $authors .= ", " if ( $authors ) ;
    $authors .= join( '',
		      "<a href=\"mailto:",
		      $email,
		      "\">",
		      $authors{"$email"},
		      "</a>" ) ;
  }
  $self->addtocontent("<font size=\"-2\">Copyright ",
		      getCopyrightDate(),
		      " &copy ",
		      $authors,
		      "</font>\n",
		      "</center>\n",
		      "</body></html>\n") ;
}

=pod

=item * listoffiles(FILE,COMMITFLAG,FILES)

Fills the mail body with the list of files.
If a filename starts with a tabulation, it is
a filename, otherwhise it is a comment.
Takes 3 args:

=over

=item FILE is the directory where the action was done (scalar)

=item COMMITFLAG indicates if the files was commit or not.

=item FILES is the associative array of the files to display

=cut
sub listoffiles($$\%) : method {
  my $self = shift ;
  my $directory = shift ;
  my $iscommit = shift ;
  my ($cnt_a,$cnt_d,$cnt_c) = ('','','') ;

  while( my ($filename,$def) = each(%{$_[0]}) ) {
    my $file = File::Spec->catfile("$directory","$filename") ;
    chomp($file) ;
    if ($file) {
      my $cnt = '' ;
      if ($iscommit) {
	my $link = $file ;
 	$link =~ s/[^a-zA-Z0-9_\-]/_/g ;
 	$cnt .= join( '', "<li><a href='#$link'>",
 		      $file,
 		      "</a></li>\n" ) ;
      }
      else {
 	$cnt .= join( '', "<li>",$file,"</li>\n" ) ;
      }

      if (!$def->{'new'}) {
	$cnt_d .= $cnt ;
      }
      elsif (!$def->{'old'}) {
	$cnt_a .= $cnt ;
      }
      else {
	$cnt_c .= $cnt ;
      }
    }
  }
  if (($cnt_a)||($cnt_d)||($cnt_c)) {
    $self->addtocontent("<table cellpadding='1' cellspacing='0' border='1' width='100%'>\n",
 			"<tr><td valign='top'>\n") ;
    if ( $cnt_c ) {
      $self->addtocontent("<strong>Modified files:</strong><ul>",
			  $cnt_c,
			  "</ul>") ;
    }
    if ( $cnt_a ) {
      $self->addtocontent("<strong>Added files:</strong><ul>",
			  $cnt_a,
			  "</ul>") ;
    }
    if ( $cnt_d ) {
      $self->addtocontent("<strong>Removed files:</strong><ul>",
			  $cnt_d,
			  "</ul>") ;
    }
    $self->addtocontent("</td></tr>\n",
 			"</table><br>\n") ;
  }
}

=pod

=item * logmessage(AUTHOR,EMAIL,LOG,[LOG...])

Fills the mail body with the log message.
Takes n args:

=over

=item AUTHOR is the name of the author (scalar)

=item EMAIL is the email of author (scalar)

=item LOG is one line of the log message (scalar).

=cut
sub logmessage($$@) : method {
  my $self = shift ;
  my $authorname = shift ;
  my $email = shift ;

  $self->addtocontent("<table cellpadding='1' cellspacing='0' border='1' width='100%'>\n",
		      "<tr><td valign='top' bgcolor='",
		      $self->{'colors'}{'descriptionBG'},
		      "' nowrap='true'>\n" ) ;
  if ( $authorname ) {
    $self->addtocontent("<center><u><strong>Log&nbsp;Message&nbsp;by&nbsp;",
			$authorname,
			"</strong></u></center><br>\n") ;
  }
  else {
    $self->addtocontent("<center><u><strong>Log Message</strong></u></center><br>\n") ;
  }

  foreach my $line ( @_ ) {
    chomp($line) ;
    if ( $line ) {
      $self->addtocontent("<tt>",
			  $self->text2html( $line ),
			  "</tt><br>\n" ) ;
    }
  }

  $self->addtocontent("<br>\n--<br>\n<small>by <font color=\"",
		      $self->{'colors'}{'$authorColor'},
		      "\">" ) ;
  if ( $email ) {
    $self->addtocontent("<a href='mailto:$email'>",
			$authorname,
			"</a>") ;
  }
  else {
    $self->addtocontent($authorname) ;
  }
  $self->addtocontent("</font></small></td></tr></table><br>\n\n") ;
}

=pod

=item * startfile(FILE)

Start the diff section for the specified file.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar).

=cut
sub startfile($) : method {
  my $self = shift ;
  $self->addtocontent("<table cellpadding='1' cellspacing='0' border='1' width='100%'>\n",
		      "<tr><td valign='top'>\n") ;
}

=pod

=item * stopfile(FILE)

Stop the diff section for the specified file.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar).

=cut
sub stopfile($) : method {
  my $self = shift ;
  $self->addtocontent("</td></tr>\n",
		      "</table><br>\n") ;
}

=pod

=item * filedescription(FILE,DIRECTORY,OLDVERSION,OLDDATE,NEWVERSION,NEWDATE)

Fills the mail body with the description of the specified file.
Takes 6 args:

=over

=item FILE is the name of the file (scalar).

=item DIRECTORY is the directory where the file was (scalar).

=item OLDVERSION is the previous version of the file in CVS (scalar or undef).

=item OLDDATE is the date of the previous version (scalar or undef).

=item NEWVERSION is the current version of the file in CVS (scalar or undef).

=item NEWDATE is the date of the current version (scalar or undef).

=cut
sub filedescription($$$$$$) : method {
  my $self = shift ;
  my $filename = shift ;
  my $directory = shift ;
  my $oldversion = shift ;
  my $olddate = shift ;
  my $newversion = shift ;
  my $newdate = shift ;

  $self->addtocontent("<table cellpadding='1' cellspacing='0' border='1' width='98%' bgcolor='",
		      $self->{'colors'}{'fileHeadRegion'},
		      "' align='Center'>\n",
		      "<tr><td valign='top' nowrap='true'><tt>\n") ;
  chomp($filename) ;
  my $link = File::Spec->catfile($directory,$filename) ;
  chomp($link) ;
  $link =~ s/[^a-zA-Z0-9_\-]/_/g ;

  $self->addtocontent("<center><strong><a name='",
		      $link,
		      "'>",
		      File::Spec->catfile($directory,$filename),
		      "</a></strong></center><br>\n") ;

  if (!$oldversion) {
    $self->addtocontent("added file, no diff available<br>\n") ;
  }
  elsif (!$newversion) {
    $self->addtocontent("removed file, no diff available<br>\n") ;
  }
  else {
    $self->addtocontent( "Previous version: ",
			 $self->text2html($oldversion),
			 ", ",
			 $self->text2html($olddate),
			 "<br>\n",
			 "Current version: ",
			 $self->text2html($newversion),
			 ", ",
			 $self->text2html($newdate),
			 "\n" ) ;
  }
  $self->addtocontent("</tt></td></tr>\n",
		      "</table>\n") ;
}

=pod

=item * startfilediff(FILE,HEAD)

Start the diff for the specified file.
Takes 2 args:

=over

=item FILE is the name of the file (scalar).

=item HEAD is the text which describe this diff (scalar).

=cut
sub startfilediff($$) : method {
  my $self = shift ;
  my $filename = shift ;

  chomp($_[0]) ;
  my @lines = split(/\s+/,$_[0]) ;
  my $from = $lines[0];
  my $to = $lines[1] ;

  $self->addtocontent("<br>\n",
		      "<table cellpadding='1' cellspacing='0' border='1' width='98%' align='Center'>\n",
		      "<tr><td valign='top' bgcolor='",
		      $self->{'colors'}{'diffHeadRegion'},
		      "' nowrap='true'><tt>From line:&nbsp;",
		      $self->text2html($from),
		      "&nbsp;&nbsp;To line:&nbsp;",
		      $self->text2html($to),
		      "</tt></td></tr><tr><td valign='top' nowrap='true'><tt>\n") ;
}

=pod

=item * stopfilediff(FILE)

Stop the diff for the specified file.
Takes 1 arg:

=over

=item FILE is the name of the file (scalar).

=cut
sub stopfilediff($) : method {
  my $self = shift ;
  $self->addtocontent("</tt></td></tr>\n",
		      "</table>\n") ;
}

=pod

=item * diff_diff(FILE,TEXT)

Add a diff message which describes a change for the specified file.
Takes 2 args:

=over

=item FILE is the name of the file (scalar).

=item TEXT is the diff text to add (scalar).

=cut
sub diff_diff($$) : method {
  my $self = shift ;
  $self->addtocontent("&nbsp;&nbsp;",
		      $self->text2html($_[1]),
		      "<br>\n") ;
}

=pod

=item * diff_del(FILE,TEXT)

Add a diff message which describes a deletion for the specified file.
Takes 2 args:

=over

=item FILE is the name of the file (scalar).

=item TEXT is the diff text to add (scalar).

=cut
sub diff_del($$) : method {
  my $self = shift ;
  $self->addtocontent("<font color='",
		      $self->{'colors'}{'diffDelColor'},
		      "'>-&nbsp;",
		      $self->text2html($_[1]),
		      "</font><br>\n");
}

=pod

=item * diff_add(FILE,TEXT)

Add a diff message which describes an addition for the specified file.
Takes 2 args:

=over

=item FILE is the name of the file (scalar).

=item TEXT is the diff text to add (scalar).

=cut
sub diff_add($$) : method {
  my $self = shift ;
  $self->addtocontent("<font color='",
		      $self->{'colors'}{'diffAddColor'},
		      "'>+&nbsp;",
		      $self->text2html($_[1]),
		      "</font><br>\n" );
}

1;
__END__

=back
