# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

=pod

=head1 NAME

SendDiff::NotificationDaemon - Daemon for the senddiff script

=head1 DESCRIPTION

SendDiff::NotificationDaemon is a Perl module which permits to launch
the notification daemon of senddiff.

=cut
package SendDiff::NotificationDaemon;
@ISA = ('Exporter');
@EXPORT = qw() ;
@EXPORT_OK = qw();
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
my $VERSION = "0.1" ;

use File::Spec ;
use SendDiff::Email ;
use SendDiff::Error ;

=pod

=back

=head1 CONSTRUCTOR DESCRIPTION

This section contains only the constructor in Deamon.pm itself.

=over

=item * new(DIRECTORY,METHOD,[PARAM])

Constructs a factory.
Takes 3 args:

=over

=item DIRECTORY is the path to the spool directory (scalar).

=item METHOD is the method identifier (scalar). It must be one of I<smtp>, I<sendmail> and I<mail>.

=item PARAM is the parameter (scalar) which corresponds to the desired method: the SMTP server's address, the sendmail binary path.

=back

=cut
sub new($$;$): method {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self ;
  if (ref($proto)) {
    %{$self} = %{$proto} ;
  }
  else {
    $self = { 'spool' => $_[0] || '/var/spool/senddiff',
	      'mail-method' => $_[1] || 'mail',
	      'mail-param' => $_[2],
	    } ;
  }
  bless( $self, $class );
  return $self;
}

=pod

=back

=head1 METHOD DESCRIPTIONS

This section contains only the methods in NotificationDaemon.pm itself.

=over

=item * run()

Run this daemon.

=cut
sub run() : method {
  my $self = shift ;
  if ( $self->{'spool'} ) {
    if ( ! -d "$self->{'spool'}" ) {
      safeQuit("$self->{'spool'}: $!") ;
    }
    elsif ( ! -r "$self->{'spool'}" ) {
      safeQuit("$self->{'spool'}: $!") ;
    }
    elsif ( ! -w "$self->{'spool'}" ) {
      safeQuit("$self->{'spool'}: $!") ;
    }

    my $tries = 0 ;
    while ( ($tries<10) && ( my @files = $self->_readspool() ) ) {
      foreach my $file (@files) {
	my $filename = File::Spec->catfile("$self->{'spool'}","$file") ;
	if ( -r "$filename" ) {
	  $self->_sendfromfile("$filename") ;
	  unlink("$filename") or
	    safeQuit("$filename: $!");
	}
      }
      $tries ++ ;
    }
  }
  return 1 ;
}

=pod

=item * _readspool()

Replies the list of the files inside the spool.

=cut
sub _readspool() : method {
  my $self = shift ;
  my @files = () ;
  local *DIR ;
  opendir(*DIR,"$self->{'spool'}")
    or safeQuit("$self->{'spool'}: $!") ;
  while (my $file = readdir(*DIR)) {
    if ($file =~ /^notify/) {
      push @files, "$file" ;
    }
  }
  closedir(*DIR) ;
  return @files ;
}

=pod

=item * _sendfromfile(FILE)

Send an email for which the content was inside the specified file.
Takes 1 arg:

=over

=item FILE is the filename (scalar).

=back

=cut
sub _sendfromfile($) : method {
  my $self = shift ;
  my $filename = shift ;

  local *FID ;
  my @recipients = () ;
  my @lines = () ;
  my $subject = '' ;
  my $from = '' ;

  open(*FID,"< $filename")
    or safeQuit("$filename: $!") ;
  while (my $line = <FID> ) {
    if ((!@recipients)&&($line =~ /^To:\s*(.+)\s*$/i)) {
      @recipients = split /\s*,\s*/, "$1" ;
    }
    elsif ((!$from)&&($line =~ /^From:\s*(.+)\s*$/i)) {
      $from = "$1" ;
    }
    elsif ((!$subject)&&($line =~ /^Subject:\s*(.+)\s*$/i)) {
      $subject = "$1" ;
    }
    else {
      push @lines, $line ;
    }
  }
  close(*FID) ;

  my $mailer = initMailer("$from", $self->{'mail-method'}, $self->{'mail-param'}) ;
  if ( $mailer ) {
    $mailer->setRecipients(@recipients) ;

    if ( $mailer->hasRecipients() ) {

      $mailer->sendemail("$subject", join('',@lines)) ;

    }

  }
}

1;
__END__

=back
