#!/usr/bin/perl -w

# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use strict ;

use Getopt::Long ;
use File::Spec ;
use File::Basename ;

#------------------------------------------------------
#
# Initialization code
#
#------------------------------------------------------
my $PERLSCRIPTDIR ;
BEGIN{
  # Where is this script?
  $PERLSCRIPTDIR = __FILE__;
  my $scriptdir = dirname( $PERLSCRIPTDIR );
  while ( -e $PERLSCRIPTDIR && -l $PERLSCRIPTDIR ) {
    $PERLSCRIPTDIR = readlink($PERLSCRIPTDIR);
    if ( substr( $PERLSCRIPTDIR, 0, 1 ) eq '.' ) {
      $PERLSCRIPTDIR = File::Spec->catfile( $scriptdir, "$PERLSCRIPTDIR" ) ;
    }
    $scriptdir = dirname( $PERLSCRIPTDIR );
  }
  $PERLSCRIPTDIR = dirname( $PERLSCRIPTDIR ) ;
  $PERLSCRIPTDIR = File::Spec->rel2abs( "$PERLSCRIPTDIR" );
  # Push the path where the script is to retreive the arakhne.org packages
  push(@INC,"$PERLSCRIPTDIR");

}

use SendDiff::Release ;
use SendDiff::Error ;
use SendDiff::Email ;
use SendDiff::Cvs ;
use SendDiff::Cvs::User ;
use SendDiff::Cvs::Run ;
use SendDiff::Cvs::Log ;

my $DEFAULT_FACTORY = 'HTML' ;
my %cmdline_options = ( 'verbose' => 1,
		      ) ; # command line arguments

#------------------------------------------------------
#
# FUNCTIONS
#
#------------------------------------------------------

#------------------------------------------------------
# Display the list of available factories
sub display_factories() {
  my @pack = ( 'SendDiff', 'Factory' ) ;
  @pack = ( File::Spec->splitdir($PERLSCRIPTDIR), @pack ) ;
  foreach my $file ( glob(File::Spec->catfile(@pack, '*.pm') ) ) {
    my $name = basename($file) ;
    if ( $name =~ /^(.+)\.pm$/ ) {
      $name = $1 ;
      print join( '',
 		  "$name",
 		  ( $DEFAULT_FACTORY && ( $DEFAULT_FACTORY eq $name ) ) ?
 		  " (default)" : "",
 		  "\n" ) ;
    }
  }
}

#------------------------------------------------------
# Replies the factory
sub initialize_factory() {
  verbose("\tinitializing the factory...", 2);
  my $pack = $DEFAULT_FACTORY ;

  if ( $cmdline_options{'factory'} ) {
    $pack = $cmdline_options{'factory'} ;
  }

  #
  # Get the package name and include it.
  #

  # try filesystem path
  eval "require '$pack';" ;
  if ( ! $@ ) {
    # the package was a file, extract the package name
    local *FID ;
    if ( open(*FID, "< $pack") ) {
      my @lines = <FID> ;
      close(*FID) ;
      my @r = grep(/^\s*package\s+[^;]+\s*;/,@lines) ;
      $pack = undef ;
      if ( @r ) {
	$r[0] =~ /^\s*package\s+([^;]+)\s*;/ ;
	$pack = "$1" ;
      }
      if (!$pack) {
	safeWarning( "the factory pm '$pack' has not valid, use default factory.\n$!\n" ) ;
	$pack = $DEFAULT_FACTORY ;
      }
    }
    else {
	safeWarning( "unable to open the factory pm '$pack', use default factory.\n$!\n" ) ;
	$pack = $DEFAULT_FACTORY ;
    }
  }
  else {
    # try perl package name
    eval "require $pack;" ;
    if ( $@ ) {
      # try SendDiff package
      $pack = "SendDiff::Factory::".$pack ;
      eval "require $pack;" ;
      if ( $@ ) {
	safeWarning( "unable to find the factory pm '$pack', use default factory.\n$@\n" ) ;
	$pack = $DEFAULT_FACTORY ;
      }
    }
  }

  #
  # Create the factory
  #
  my $factory ;
  eval '$factory = '.$pack.'->new() ;' ;
  if ( $@ ) {
    safeWarning( "unable to find the factory pm '$pack', use default factory.\n$@\n" ) ;
    require SendDiff::Factory::HTML ;
    $factory = SendDiff::Factory::HTML->new() ;
  }

  return $factory ;
}

#------------------------------------------------------
# Display the version
sub display_version() {
  print STDOUT "senddiff ".getVersionNumber().", ".getVersionDate()."\n" ;
  my $copyright = "Copyright ".getCopyrightDate()." (c) " ;
  print STDOUT $copyright ;
  my $indent = '';
  for(my $i=0;$i<length($copyright);$i++) {
    $indent .= ' ';
  }
  my %authors = getAuthors() ;
  my @emails = keys %authors ;
  if ( @emails ) {
    my $email = shift @emails ;
    print STDOUT $authors{"$email"}." <$email>" ;
    while (@emails) {
      my $email = shift @emails ;
      print STDOUT ",\n$indent".$authors{"$email"}." <$email>" ;
    }
  }
  print STDOUT "\n          under GNU GPL\n" ;
  my %contribs = getContributors() ;
  if ( %contribs ) {
    my @emails = keys %contribs ;
    print STDOUT "Contributor".((@emails>1)?'s':'').":\n" ;
    foreach my $email (@emails) {
      print STDOUT "   ".$contribs{"$email"}." <$email>\n" ;
    }
  }
}

#------------------------------------------------------
# Display the manual
sub display_manual($) {
  use Pod::Perldoc ;
  my $pd = Pod::Perldoc->new() ;
  $pd->{'args'} = [ $_[0] ] ;
  $pd->process() ;
}

#------------------------------------------------------
# Initialize the parameters
sub initialize_parameters() {
  verboselevel($cmdline_options{'verbose'}) ;
  verbose("\tinitializing the parameters...", 2);

  setCVSROOT($cmdline_options{'cvsroot'}) ;

  # check for the CVS binary and the CVSROOT
  if ( $cmdline_options{'cvs-binary'} ) {
    setCVSBinary($cmdline_options{'cvs-binary'}) ;
  }
  else {
    setCVSBinary(getCVSBinary()) ;
  }
  safeQuit("unable to find the CVS binary: ".getCVSBinary())
    unless ( -f getCVSBinary() ) ;
  safeQuit("the specified CVS binary can't be ran: ".getCVSBinary())
    unless ( -x getCVSBinary() ) ;

  # Initialize the differed notification
  if ((exists($cmdline_options{'mail'}{'differed'}))&&
      (!$cmdline_options{'mail'}{'differed'})) {
    $cmdline_options{'mail'}{'differed'} = '/var/spool/senddiff' ;
  }
}

#------------------------------------------------------
# Initialize the mailer
sub initialize_mailer() {
  verbose("\tinitializing the mailer...", 2);
  if ( exists $cmdline_options{'mail'}{'differed'} ) {
    return initMailer(($cmdline_options{'sender-email'} || $ENV{'USER'}),
		      'spool',$cmdline_options{'mail'}{'differed'}) ;
  }
  elsif ( $cmdline_options{'mail'}{'smtp'} ) {
    return initMailer(($cmdline_options{'sender-email'} || $ENV{'USER'}),
		      'smtp',$cmdline_options{'mail'}{'smtp'}) ;
  }
  elsif ( exists $cmdline_options{'mail'}{'sendmail'} ) {
    return initMailer(($cmdline_options{'sender-email'} || $ENV{'USER'}),
		      'sendmail',$cmdline_options{'mail'}{'sendmail'}) ;
  }
  else {
    return initMailer(($cmdline_options{'sender-email'} || $ENV{'USER'}),
		      'mail');
  }
}

#------------------------------------------------------
# Replies the CVS users' data
sub read_cvsusers() {
  verbose("\textracting the cvs users' data...", 2);
  if ( ! $cmdline_options{'user-file'} ) {
    my $root = getLocalCVSROOT() ;
    if ( $root ) {
      $cmdline_options{'user-file'} = getCVSUserFilename("$root") ;
    }
  }
  return getCVSUserFile($cmdline_options{'user-file'},$cmdline_options{'user-group'}) ;
}

#------------------------------------------------------
# Initialize the recipient emails' list
sub initialize_recipients($\%) {
  verbose("\textracting the recipients' email...", 2);
  my @emails = () ;
  if ( $cmdline_options{'receiver-email'} ) {
    @emails = split( /\s*:\s*/,$cmdline_options{'receiver-email'} ) ;
  }
  else {
    my %rev = reverse %{$_[1]} ;
    @emails = keys %rev ;
  }

  $_[0]->setRecipients(@emails);
}

#------------------------------------------------------
#
# MAIN
#
#------------------------------------------------------

#------------------------------------------------------
# Read the command line
#
Getopt::Long::Configure("bundling") ;
if ( ! GetOptions( "version"     => \$cmdline_options{'version'},
		   "help|h|?"    => \$cmdline_options{'help'},
		   "man|manual"  => \$cmdline_options{'manual'},
		   "q|quiet"     => sub { $cmdline_options{'verbose'} = 0; },
		   'v+'          => \$cmdline_options{'verbose'},
		   "users=s"     => \$cmdline_options{'user-file'},
		   "usergroup=s" => \$cmdline_options{'user-group'},
		   "email=s"     => \$cmdline_options{'receiver-email'},
		   "sender=s"    => \$cmdline_options{'sender-email'},
		   "log=s"       => \$cmdline_options{'cvs-log'},
		   "cvs=s"       => \$cmdline_options{'cvs-binary'},
		   "cvsroot=s"   => \$cmdline_options{'cvsroot'},
		   "sendmail:s"  => sub { $cmdline_options{'mail'} = {sendmail=>$_[1]}; },
		   "smtp=s"      => sub { $cmdline_options{'mail'} = {smtp=>$_[1]}; },
		   'd|diff:s'    => sub { $cmdline_options{'mail'} = {differed=>$_[1]}; },
		   "project=s"   => \$cmdline_options{'project-name'},
		   'factorylist' => \$cmdline_options{'factory-list'},
		   'factory=s'   => \$cmdline_options{'factory'},
		   'debug'       => \$cmdline_options{'debug'},
		  ) ) {
  use Pod::Usage;
  pod2usage(2) ;
}


#------------------------------------------------------
# Treat information parameters
#
if ( $cmdline_options{'version'} ) {
  # Show the version number
  display_version() ;
  exit 1 ;
}
elsif ( $cmdline_options{'manual'} ) {
  # Show the manual screen
  display_manual("$0") ;
  exit(1) ;
}

# Show the factory list
if ( $cmdline_options{'factory-list'} ) {
  display_factories() ;
  exit 1 ;
}

if ( $cmdline_options{'help'} || ( $#ARGV < 0 ) ) {
  use Pod::Usage;
  pod2usage(1);
}

#------------------------------------------------------
# Set the options accordingly to the command line
#
initialize_parameters() ;

#------------------------------------------------------
# Initialization
#
# Initialize the factory
my $factory = initialize_factory() ;

# Initialize the mailer
my $mailer = initialize_mailer() ;

# extract the cvs users data.
my %cvsusers = read_cvsusers() ;

# extract the recipient mail address, the adress where
# to send the diff, and the cvs argument from the
# command line
initialize_recipients($mailer,%cvsusers) ;

# Display debug info
if ($cmdline_options{'debug'}) { 
  use Data::Dumper;
  my @IN = <STDIN>;
  die(Dumper({'cmd'=>$ARGV[0],'stdin'=>\@IN}));
}

if ( $mailer->hasRecipients() ) {

  # Create the Diff page
  my $logger = SendDiff::Cvs::Log->new($factory,\%cvsusers,$cmdline_options{'cvs-log'}) ;

  $logger->generatePage($ARGV[0]) ;

  # Send email
  my $subject = $factory->getsubject() ;
  if ( $cmdline_options{'project-name'} ) {
    $subject = "[".$cmdline_options{'project-name'}."] $subject" ;
  }
  $mailer->sendemail("$subject",$factory->getcontent()) ;

}

exit 0 ;

__END__

=head1 NAME

senddiff - A CVS repository notifier


=head1 SYNOPSYS

senddiff [options] [--log I<file>] I<cvscmdline>

=head1 DESCRIPTION

senddiff is a script which permits to notify users by email
each time a CVS repository has changed.
This script is designed to be called from the CVS script
CVSROOT/loginfo. I<cvscmdline> must be a parameter passed by
the CVS server (such as "%p ${sVv}"). senddiff reads the log
from the standard input unless the option --log was passed.

If you don't specify one of the options --diff, --sendmail or
--smtp, senddiff uses Mail::Mailer to deliver emails.

=head1 EXAMPLE

You could use this script inside the file I<CVSROOT/loginfo> of a
cvs repository:

DEFAULT senddiff --sendmail --sender cvs@localhost --email root@localhost -- "%p %{sVv}"

=head1 OPTIONS

=over 4

=item B<--cvs> I<file>

Specifies the binary location of CVS (default is /usr/bin/cvs).

=item B<--cvsroot> I<file>

Sets the CVSROOT variable.

=item B<--debug>

Display the command line parameter and the standard input.

=item B<-d> [I<directory>]

=item B<--diff> [I<directory>]

*NO WARRANTY*
Does not send immediatly the notification emails. They will be sent by
the notification daemon senddiffd. If a I<directory> was
specified, it will contain the notification files.
This feature is not warranty. The senddiff daemon will not detect
the damaged or suppressed spool files.

=item B<--email> I<emails>

Set the emails of the notification receivers. I<emails> must be separated
by a column if they was more than one.

=item B<--factory> I<pm>

Force to use I<pm> as the factory of the email content. The specified I<pm>
must be a subclass of SendDiff::Factory.

=item B<--factorylist>

List the available factory's pm on this system.

=item B<--log> I<file>

Set the file from which the CVS log must be extracted.
If this option was not passed, senddiff reads the log
from the standard input.

=item B<-?>

=item B<-h>

=item B<--help>

Show the list of available options.

=item B<--man>

=item B<--manual>

Show the manual page.

=item B<--project> I<name>

Sets the I<name> of the project. It will be merge
at the start of the mail subject.

=item B<-q>

=item B<--quiet>

Don't print any message.

=item B<--sender> I<email>

Set the email of the notification sender.

=item B<--sendmail> [I<file>]

Use sendmail to deliver the emails. If I<file> was specified,
it is the path to sendmail (default is /usr/sbin/sendmail).

=item B<--smtp> I<name>

Use a SMTP server to deliver the emails. I<name> is the name
of the SMTP server.

=item B<--usergroup> I<group>

The emails must be sent to the local users which are inside the the specified I<group>.
This option uses the file $HOME/.ignore_cvs_notification to know if the user
wants to not receive the notifications. This option also uses the file
$HOME/.forward to use the right email addresses.

*CAUTION*
This feature is only for Unix operating systems.

=item B<--users> I<file>

The user's emails must be read from the I<file>.
This file must contain one line per user with the one of
the following formats:

I<name>:I<email>

=item B<--version>

Show the version of this script.

=back

=head1 PLATFORM CONSTRAINTS

This script is designed to be run on Unix platform.
We don't give any warranty for other platforms.

=head1 COPYRIGHT

Copyright (c) 1998-05 Mahdi Hannoun <mhd@arakhne.org> and
                      Stphane Galland <galland@arakhne.org>,
                      under GPL.

=head1 SEE ALSO

L<cvs>(1), L<senddiffd>(8)
