#!/usr/bin/perl -w

# SendDiff script to atomate the diff mail send on cvs commit
# Copyright (C) 2002-04  Mahdi Hannoun <mhd@arakhne.org>,
#                        Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use strict ;

use Getopt::Long ;
use File::Spec ;
use File::Basename ;

#------------------------------------------------------
#
# Initialization code
#
#------------------------------------------------------
my $PERLSCRIPTDIR ;
BEGIN{
  # Where is this script?
  $PERLSCRIPTDIR = __FILE__;
  my $scriptdir = dirname( $PERLSCRIPTDIR );
  while ( -e $PERLSCRIPTDIR && -l $PERLSCRIPTDIR ) {
    $PERLSCRIPTDIR = readlink($PERLSCRIPTDIR);
    if ( substr( $PERLSCRIPTDIR, 0, 1 ) eq '.' ) {
      $PERLSCRIPTDIR = File::Spec->catfile( $scriptdir, "$PERLSCRIPTDIR" ) ;
    }
    $scriptdir = dirname( $PERLSCRIPTDIR );
  }
  $PERLSCRIPTDIR = dirname( $PERLSCRIPTDIR ) ;
  $PERLSCRIPTDIR = File::Spec->rel2abs( "$PERLSCRIPTDIR" );
  # Push the path where the script is to retreive the arakhne.org packages
  push(@INC,"$PERLSCRIPTDIR");

}

use SendDiff::Release ;
use SendDiff::Error ;
use SendDiff::NotificationDaemon ;

my %cmdline_options = () ; # command line arguments

#------------------------------------------------------
#
# FUNCTIONS
#
#------------------------------------------------------

#------------------------------------------------------
# Display the version
sub display_version() {
  print STDOUT "senddiffd ".getVersionNumber().", ".getVersionDate()."\n" ;
  my $copyright = "Copyright ".getCopyrightDate()." (c) " ;
  print STDOUT $copyright ;
  my $indent = '';
  for(my $i=0;$i<length($copyright);$i++) {
    $indent .= ' ';
  }
  my %authors = getAuthors() ;
  my @emails = keys %authors ;
  if ( @emails ) {
    my $email = shift @emails ;
    print STDOUT $authors{"$email"}." <$email>" ;
    while (@emails) {
      my $email = shift @emails ;
      print STDOUT ",\n$indent".$authors{"$email"}." <$email>" ;
    }
  }
  print STDOUT "\n          under GNU GPL\n" ;
  my %contribs = getContributors() ;
  if ( %contribs ) {
    my @emails = keys %contribs ;
    print STDOUT "Contributor".((@emails>1)?'s':'').":\n" ;
    foreach my $email (@emails) {
      print STDOUT "   ".$contribs{"$email"}." <$email>\n" ;
    }
  }
}

#------------------------------------------------------
# Display the manual
sub display_manual($) {
  use Pod::Perldoc ;
  my $pd = Pod::Perldoc->new(section=>8) ;
  $pd->{'args'} = [ $_[0] ] ;
  $pd->process() ;
}

#------------------------------------------------------
# Initialize the parameters
sub initialize_parameters() {
  verboselevel(0) ;
  if ( ! $cmdline_options{'spool'} ) {
    $cmdline_options{'spool'} = '/var/spool/senddiff' ;
  }
}

#------------------------------------------------------
# Initialize the daemon
sub initialize_daemon() {
  if ( $cmdline_options{'mail'}{'smtp'} ) {
    return SendDiff::NotificationDaemon->new($cmdline_options{'spool'},
					     'smtp',$cmdline_options{'mail'}{'smtp'}) ;
  }
  elsif ( $cmdline_options{'mail'}{'sendmail'} ) {
    return SendDiff::NotificationDaemon->new($cmdline_options{'spool'},
					     'sendmail',$cmdline_options{'mail'}{'sendmail'}) ;
  }
  else {
    return SendDiff::NotificationDaemon->new($cmdline_options{'spool'},
					     'mail');
  }
}

#------------------------------------------------------
#
# MAIN
#
#------------------------------------------------------

#------------------------------------------------------
# Read the command line
#
Getopt::Long::Configure("bundling") ;
if ( ! GetOptions( "version"     => \$cmdline_options{'version'},
		   "help|h|?"    => \$cmdline_options{'help'},
		   "man|manual"  => \$cmdline_options{'manual'},
		   "sendmail:s"  => sub { $cmdline_options{'mail'} = {sendmail=>$_[1]}; },
		   "smtp=s"      => sub { $cmdline_options{'mail'} = {smtp=>$_[1]}; },
		   'spool=s'     => \$cmdline_options{'spool'},
		  ) ) {
  use Pod::Usage;
  pod2usage(2) ;
}

#------------------------------------------------------
# Treat information parameters
#
if ( $cmdline_options{'version'} ) {
  # Show the version number
  display_version() ;
  exit 1 ;
}
elsif ( $cmdline_options{'manual'} ) {
  # Show the manual screen
  display_manual("$0") ;
  exit(1) ;
}

if ( $cmdline_options{'help'} || ( $#ARGV > 0 ) ) {
  use Pod::Usage;
  pod2usage(1);
}

#------------------------------------------------------
# Set the options accordingly to the command line
#
initialize_parameters() ;

#------------------------------------------------------
# Initialization
#
my $daemon = initialize_daemon() ;

#------------------------------------------------------
# Running
#
$daemon->run() ;

exit 0 ;

__END__

=head1 NAME

senddiffd - A CVS repository notifier daemon


=head1 SYNOPSYS

senddiffd [options]

=head1 DESCRIPTION

senddiff is a script which permits to notify users by email
each time a CVS repository has changed.

senddiffd is the daemon which sends the emails.

*NO WARRANTY*
This daemon is not warranty. It will not detect
the damaged or suppressed spool files.

If you don't specify the option --sendmail or --smtp, senddiffd
uses Mail::Mailer to deliver emails.

=head1 OPTIONS

=over 4

=item B<-?>

=item B<-h>

=item B<--help>

Show the list of available options.

=item B<--man>

=item B<--manual>

Show the manual page.

=item B<--sendmail> [I<file>]

Use sendmail to deliver the emails. If I<file> was specified,
it is the path to sendmail (default is /usr/sbin/sendmail).

=item B<--smtp> I<name>

Use a SMTP server to deliver the emails. I<name> is the name
of the SMTP server.

=item B<--spool> I<directory>

Specifies the name of the spool directory (by default /var/spool/senddiff).

=item B<--version>

Show the version of this script.

=back

=head1 PLATFORM CONSTRAINTS

This script is designed to be run on Unix platform.
We don't give any warranty for other platforms.

=head1 COPYRIGHT

Copyright (c) 1998-05 Mahdi Hannoun <mhd@arakhne.org> and
                      Stphane Galland <galland@arakhne.org>,
                      under GPL.

=head1 SEE ALSO

L<cvs>(1), L<senddiff>(1)
